import ESLintWebpackPlugin from "eslint-webpack-plugin";
import path from "path";
import StylelintPlugin from "stylelint-webpack-plugin";
import ConfigHelper from "../libs/ConfigHelper.js";
function getEnvDef(isDebugMode) {
    return isDebugMode ? "development" : "production";
}
function getPlugins(isDebugMode) {
    const plugins = [];
    const defaultStylelintConf = {
        extensions: ["css", "less", "scss", "sass"],
        files: "./src",
    };
    let stylelintConf = ConfigHelper.get("stylelint", true);
    if (stylelintConf === true) {
        stylelintConf = defaultStylelintConf;
    }
    if (stylelintConf) {
        plugins.push(new StylelintPlugin(stylelintConf));
    }
    const defaultESLintConf = {
        files: "./src",
        extensions: ["ts", "tsx", "js"],
        failOnError: !isDebugMode,
    };
    let esLintPluginConf = ConfigHelper.get("eslint", true);
    if (esLintPluginConf === true) {
        esLintPluginConf = defaultESLintConf;
    }
    if (esLintPluginConf) {
        plugins.push(new ESLintWebpackPlugin(esLintPluginConf));
    }
    return plugins;
}
function getRules() {
    const rules = [];
    rules.push({
        exclude: /[\\/]node_modules[\\/]|\.d\.ts$/i,
        test: /\.tsx?$/i,
        use: [
            {
                loader: "babel-loader",
            },
            {
                loader: "ts-loader",
            },
        ],
    });
    return rules;
}
export default function base(isDebugMode) {
    return {
        mode: getEnvDef(isDebugMode),
        devtool: isDebugMode ? "source-map" : undefined,
        plugins: getPlugins(isDebugMode),
        module: {
            rules: getRules(),
        },
        resolve: {
            extensions: [".ts", ".tsx", ".js", ".jsx", ".css", ".png", ".jpg", ".gif", ".less", "sass", "scss", "..."],
            modules: [
                "node_modules",
                path.resolve("./src"),
            ],
        },
        resolveLoader: {
            modules: [
                "node_modules",
            ],
        },
    };
}
