import fs from "fs-extra";
import Path from "path";
import Logger from "../libs/Logger.js";
const log = Logger("ConfigHelper");
const configureJSON = Path.resolve("./config/configure.json");
const appConfJSON = Path.resolve("./config/app.json");
export default class ConfigHelper {
    static store = {};
    static privateGet(node, defaultValue = null, configPath = configureJSON) {
        let result = defaultValue;
        const key = `${configPath}->${node}`;
        log.info("ConfigHelper privateGet key: ", key);
        if (ConfigHelper.store[`${configPath}-${node}`]) {
            log.info("read in store", key + "->" + JSON.stringify(ConfigHelper.store[`${configPath}-${node}`]));
            return ConfigHelper.store[`${configPath}-${node}`];
        }
        log.info("confighelper privateGet configPath: ", configPath);
        try {
            let store = fs.readJSONSync(configPath);
            const info = node.split(".");
            log.info("ConfigHelper.get.info", info);
            for (const item of info) {
                log.info("item", item);
                log.info("store !== null", store !== null);
                log.info("store[item]", store[item]);
                if (store !== null && typeof store[item] !== "undefined") {
                    store = store[item];
                }
                else {
                    store = null;
                    break;
                }
            }
            if (store !== null) {
                log.info("ConfigHelper.get.store", store);
                result = store;
            }
            else {
                log.info("ConfigHelper.get.store get empty value");
            }
        }
        catch (error) {
            log.info("ConfigHelper.get raise an error: ", key + " -> " + error.message);
        }
        if (result === null) {
            const msg = `ConfigHelper > 未能获取到缺省和默认配置,${configPath},${node}。`;
            throw new Error(msg);
        }
        log.info("ConfigHelper.get", key + "->" + JSON.stringify(result));
        ConfigHelper.store[`${configPath}-${node}`] = result;
        return result;
    }
    static set(node, value, configPath = configureJSON) {
        ConfigHelper.store[`${configPath}-${node}`] = value;
    }
    static get(node, defaultValue = null, configPath = configureJSON) {
        return ConfigHelper.privateGet(node, defaultValue, configPath);
    }
    static getPackageName() {
        return ConfigHelper.get("name", null, appConfJSON);
    }
    static getAssetsPathPrefix() {
        let prefix = ConfigHelper.get("assetsPathPrefix", "", appConfJSON);
        if (prefix && !prefix.endsWith("/")) {
            prefix += "/";
        }
        return prefix;
    }
    static getCDN() {
        return ConfigHelper.get("cdn", "", appConfJSON);
    }
    static getPublicPath() {
        return `${ConfigHelper.getAssetsPathPrefix()}`;
    }
}
