import { cyan, green, red } from "colorette";
import ora from "ora";
import path from "path";
import Logger from "./Logger.js";
const log = Logger("TaskBase");
export class TaskBase {
    watcher;
    count = 1;
    taskName = "TaskBase";
    isWatchMode = false;
    isDebugMode = false;
    isAnalyzMode = false;
    rootPath = "./";
    src;
    dist;
    state = false;
    constructor(name) {
        this.taskName = name;
        this.src = path.resolve("./src");
        this.dist = path.resolve("./dist");
    }
    async compile(config) {
        return Promise.resolve(log.warn(`${cyan(this.taskName)} 未重写compile方法`));
    }
    async done() {
        return Promise.resolve(console.log(green(`${cyan(this.taskName)} task completed.\n`)));
    }
    setWatchMode(isWatchMode) {
        this.isWatchMode = isWatchMode;
        return this;
    }
    setDebugMode(isDebugMode) {
        this.isDebugMode = isDebugMode;
        return this;
    }
    setAnalyzMode(isAnalyzMode) {
        this.isAnalyzMode = isAnalyzMode;
        return this;
    }
    async run(...args) {
        const spinner = !this.isDebugMode && ora();
        try {
            spinner && spinner.start(`${this.taskName} compile task begin ...\r\n`);
            await this.compile(...args);
            spinner && spinner.succeed(`${this.taskName} compile task end.\r\n`);
        }
        catch (error) {
            spinner && spinner.fail(`${this.taskName} compile task ${red("fail")}.\r\n`);
            log.error(red(`${cyan(this.taskName)}  error: ${error.message}`), error);
        }
    }
}
export default TaskBase;
