import MiniCssExtractPlugin from "mini-css-extract-plugin";
import path from "path";
import sharePlugin from "../config/share.plugin.js";
import getGlobalConfig from "../libs/getGlobalConfig.js";
import Logger from "../libs/Logger.js";
import { WebpackTaskBase } from "../libs/WebpackTaskBase.js";
import { HelperTask } from "./HelperTask.js";
const log = Logger("ModuleFederatePack");
export class ModuleFederatePack extends WebpackTaskBase {
    globalConfig;
    constructor(taskName = "ModuleFederatePack") {
        super(taskName);
        this.globalConfig = getGlobalConfig();
        this.dist = path.resolve(`${this.rootPath}${this.globalConfig.clientOutput}/federate`);
    }
    async compile() {
        log.info("->", "ModuleFederatePack", HelperTask.taking());
        const config = {
            entry: { "index": ["raf/polyfill"], },
            output: {
                publicPath: "auto",
                path: this.dist,
            },
            name: this.taskName,
            plugins: this.getPlugins(),
            optimization: {
                splitChunks: false,
            },
        };
        log.info("pack", { config: JSON.stringify(config) });
        await super.compile(config);
    }
    getPlugins() {
        const plugins = [];
        plugins.push(new MiniCssExtractPlugin({
            filename: "[name]_[contenthash:8].css",
        }));
        plugins.push(...sharePlugin.exposeMfPlugin);
        return plugins;
    }
}
export default ModuleFederatePack;
