import { bold, green } from "colorette";
import spawn from 'cross-spawn';
import Logger from "../libs/Logger.js";
const log = Logger("RunServer");
const RUNNING_REGEXP = /server start successful, listening at port: (\d+)/;
let server;
export default async function RunServer(serverPath, debug, cb = false) {
    return new Promise(resolve => {
        let cbIsPending = !!cb;
        function onStdOut(data) {
            const time = new Date().toTimeString();
            const match = data.toString("utf8").match(RUNNING_REGEXP);
            if (match) {
                server.stdout.removeListener("data", onStdOut);
                console.log();
                console.log(bold(green("server start successful, listen at port: " + match[1])));
                console.log();
                server.stdout.on("data", data => process.stdout.write(data));
                if (typeof cb === "function") {
                    cbIsPending = false;
                    cb(parseInt(match[1], 10));
                }
                resolve();
            }
            else {
                process.stdout.write(data);
            }
        }
        if (server) {
            log.warn("server 进程即将 退出并重启...");
            server.kill("SIGINT");
        }
        const params = [];
        if (debug > 0) {
            params.push("--inspect=" + debug);
        }
        params.push(serverPath);
        log.info("启动debug server模式的服务", params);
        server = spawn("node", params, {
            env: {
                ...process.env,
                NODE_ENV: "development",
            },
            cwd: "dist",
        });
        if (cbIsPending) {
            server.once("exit", (code, signal) => {
                if (cbIsPending) {
                    throw new Error(`Server terminated unexpectedly with code: ${code} signal: ${signal}`);
                }
                else {
                    log.warn("进程重启");
                }
            });
        }
        server.stdout.on("data", onStdOut);
        server.stderr.on("data", data => process.stderr.write(data));
    });
}
process.on("exit", () => {
    if (server) {
        log.warn("server进程退出");
        server.kill("SIGINT");
        server = null;
    }
});
